#include "cluster.h"
#include "otree.h"

int main(int argc, char * argv[])
{
  // linkage: 0: max. 1: ave
  int i, j, nele, maxclus, linkage = 0;
  double ogap = 0, ** distmat;
  char crite;
  dblvec threshs;
  vector<Onode> otree;
  vector<pair<int, intset> > clusters;
  ofstream outFile, dendFile;
  ReadOption(argc, argv, crite, threshs, ogap, maxclus, distmat, nele, linkage);
  PrintOption(cout, argc, argv, crite, threshs, ogap, maxclus, distmat, nele, linkage);
  if (linkage == 0) {
    pmlotree(nele, maxclus, ogap, distmat, otree);
  } else if (linkage == 1) {
    Mylib::OutFopen(dendFile, argv[1]);
    palotree_tumat(nele, maxclus, ogap, distmat, otree, dendFile, 0);
    dendFile.close();
  } else if (linkage == 2) {
    Mylib::OutFopen(dendFile, argv[1]);
    palotree(nele, maxclus, ogap, distmat, otree, dendFile);
    dendFile.close();
  } else if (linkage == 3) {
    Mylib::OutFopen(dendFile, argv[1]);
    palotree_tumat(nele, maxclus, ogap, distmat, otree, dendFile, 1);
    dendFile.close();
  }
  Mylib::OutFopen(outFile, argv[0]);
  for (i = 0; i < threshs.size(); i++) {
    cutotree(ogap, otree, threshs[i], clusters);
    for (j = 0; j < clusters.size(); j++) {
      outFile << threshs[i] << '\t' << clusters[j].first << '\t' << clusters[j].second.size() << '\t';
      Mylib::PrintJoin(outFile, clusters[j].second, " ");
      outFile << endl;
    }
  }
  outFile.close();
  return 0;
}

void ReadOption(int & argc, char ** & argv, char & crite, dblvec & threshs, double & ogap, int & maxclus, double ** & distmat, int & nele, int & linkage)
{
  int a, b, i, j;
  double d, dmax;
  char conver = 'd', * thc, opt;
  ifstream distFile;
  maxclus = -1;

  cout << "Command line:";
  for (i = 0; i < argc; i++) {
    cout << ' ' << argv[i];
  }
  cout << endl;
  // Remove the executable name
  argv++;
  argc--;

  if (argc == 0) {
    PrintUsage(cerr);
    exit(1);
  }
  while (argv[0][0] == '-') {
    opt = argv[0][1];
    argv++;
    argc--;
    switch (opt) {
    case 'c':
      crite = argv[0][0];
      argv++;
      argc--;
      thc = strtok(argv[0], " ");
      while (thc != NULL) {
	threshs.push_back(atof(thc));
	thc = strtok(NULL, " ");
      }
      argv++;
      argc--;
      break;
    case 'g':
      ogap = atof(argv[0]);
      argv++;
      argc--;
      break;
    case 'm':
      maxclus = atoi(argv[0]);
      argv++;
      argc--;
      break;
    case 's':
      conver = argv[0][0];
      argv++;
      argc--;
      break;
    case 'l':
      if (strcmp(argv[0], "max") == 0) {
	linkage = 0;
      } else if (strcmp(argv[0], "ave") == 0) {
	linkage = 1;
      } else if (strcmp(argv[0], "ave1") == 0) {
	linkage = 2;
      } else if (strcmp(argv[0], "ave2") == 0) {
	linkage = 3;
      } else {
	cerr << "Error: linkage " << argv[0] << endl;
	PrintUsage(cerr);
	exit(1);
      }
      argv++;
      argc--;
      break;
    default:
      cerr << "Error: option " << opt << endl;
      PrintUsage(cerr);
      exit(1);
    }
  }

  nele = -g_ilarge;
  dmax = -g_dlarge;
  Mylib::InFopen(distFile, argv[0]);
  while (distFile >> a >> b >> d) {
    if (nele < a) {
      nele = a;
    }
    if (nele < b) {
      nele = b;
    }
    if (dmax < d) {
      dmax = d;
    }
  }
  distFile.close();
  nele++;
  cout << "nele\t" << nele << "\tdmax\t" << dmax << endl;
  if (maxclus == -1) {
    cout << "use nele as default for maxclus\t" << nele << endl;
    maxclus = nele;
  }
  distmat = new double *[maxclus];
  for (i = 0; i < maxclus; i++) {
    if (linkage == 0) {
      distmat[i] = new double[i + 1];
    } else {
      distmat[i] = new double[i];
    }
    for (j = 0; j < i; j++) {
      distmat[i][j] = 0;
    }
    if (linkage == 0) {
      distmat[i][i] = 0;
    }
  }
  Mylib::InFopen(distFile, argv[0]);
  argv++;
  argc--;
  while (distFile >> a >> b >> d) {
    switch (conver) {
    case 'f':
      d = dmax - d;
      break;
    }
    if (a > b) {
      distmat[a][b] = d;
    } else {
      distmat[b][a] = d;
    }
  }
}

void PrintOption(ostream & out, const int & argc, char ** const & argv, const char & crite, const dblvec & threshs, const double & ogap, const int & maxclus, double ** const & distmat, const int & nele, const int & linkage)
{
  int i;
  out << "Remaining command line:";
  for (i = 0; i < argc; i++) {
    out << ' ' << argv[i];
  }
  out << endl;
  out << "crite\t" << crite << endl;
  out << "threshs\t";
  Mylib::PrintJoin(out, threshs, " ");
  out << endl;
  out << "ogap\t" << ogap << endl;
  out << "maxclus\t" << maxclus << endl;
  out << "nele\t" << nele << endl;
  out << "linkage\t" << linkage << endl;
  if (g_debug >= 2) {
    PrintDistmat(out, distmat, nele, linkage);
  }
}

void PrintUsage(ostream & out)
{
  out << "Usage: cluster" << endl;
  out << " [-c <criterion> <thresholds>]" << endl;
  out << " [-g <ogap>]" << endl;
  out << " [-m <maxcluster>]" << endl;
  out << " [-s <conver>]" << endl;
  out << " [-l <linkage>]" << endl;
  out << " <i.3f distance>" << endl;
  out << " <o.4f out>" << endl;
  out << " <o.8f dend>" << endl;
  out << endl;
  out << "<criterion>:" << endl;
  out << " 'c': stop merging when the metric for merging" << endl;
  out << "   clusters drops below threshold" << endl;
  out << "<thresholds>:" << endl;
  out << " space-delimited thresholds. Cluster for each" << endl;
  out << endl;
  out << "<ogap>: after a cluster is first merged, within" << endl;
  out << "  <ogap>, it can still be used again" << endl;
  out << "Default: 0, don't allow overlap" << endl;
  out << endl;
  out << "<maxcluster>: # clusters may increase. When it" << endl;
  out << "  exceeds <maxcluster>, we remove clusters that are" << endl;
  out << "  merged earliest" << endl;
  out << "Default: same as # prots in <distance>. ie. #" << endl;
  out << "  clusters at beginning" << endl;
  out << "Note: <maxcluster> should > # prots in <distance>" << endl;
  out << endl;
  out << "<conver>: convert <distance> to the distance matrix" << endl;
  out << "  we use" << endl;
  out << " 'f': max - <distance>" << endl;
  out << "Default: use original <distance>" << endl;
  out << "Note: missing pairs in <distance> are treated as 0" << endl;
  out << "  after conversion" << endl;
  out << endl;
  out << "<linkage>:" << endl;
  out << " 'max': complete linkage" << endl;
  out << " 'ave': ave linkage. tumat 4 cluster pair. Exclude subset" << endl;
  out << " 'ave1': average linkage w/ touse for individual cluster" << endl;
  out << " 'ave2': ave linkage. tumat 4 cluster pair. Exclude exact match" << endl;
  out << "Default: 'max'" << endl;
  out << endl;
  out << "<distance>:" << endl;
  out << " f1: 1st prot ID starts at 0" << endl;
  out << " f2: 2nd prot ID starts at 0" << endl;
  out << " f3: distance" << endl;
  out << "Note: the order of f1 and f2 doesn't matter" << endl;
  out << "Note: missing pairs in <distance> are treated as 0" << endl;
  out << "  after conversion" << endl;
  out << "Note: # prots depends on the last prot ID in f1 and" << endl;
  out << "  f2 so be CAREFUL when trailing prots are missing" << endl;
  out << "  from <distance>" << endl;
  out << endl;
  out << "<out>:" << endl;
  out << " f1: threshold" << endl;
  out << " f2: cluster ID" << endl;
  out << " f3: size of the cluster" << endl;
  out << " f4: space-delimited cluster members" << endl;
  out << endl;
  out << "<dend>: output the dendrogram learned" << endl;
  out << " f1: cluster ID" << endl;
  out << " f2: cluster ID of left child. -1 if none" << endl;
  out << " f3: cluster ID of right child. -1 if none" << endl;
  out << " f4: distance when this cluster is created" << endl;
  out << " f5: ub of distance when the cluster is removed" << endl;
  out << " f6: lb of distance when the cluster is removed" << endl;
  out << " f7: touse when the cluster is removed" << endl;
  out << " f8: size of the cluster" << endl;
}
