#include <map>
#include "dendclus.h"
#include "otree.h"

int main(int argc, char * argv[])
{
  int i, j;
  double cutgap = 0;
  char crite;
  dblvec threshs;
  // the position of an element in otree is its cluster ID in the dendrogram
  vector<Onode> otree;
  vector<pair<int, intset> > clusters;
  ofstream outFile;
  ReadOption(argc, argv, crite, threshs, cutgap, otree);
  PrintOption(cout, argc, argv, crite, threshs, cutgap);
  Mylib::OutFopen(outFile, argv[0]);
  for (i = 0; i < threshs.size(); i++) {
    cutotree(cutgap, otree, threshs[i], clusters);
    for (j = 0; j < clusters.size(); j++) {
      outFile << threshs[i] << '\t' << clusters[j].first << '\t' << clusters[j].second.size() << '\t';
      Mylib::PrintJoin(outFile, clusters[j].second, " ");
      outFile << endl;
    }
  }
  outFile.close();
  return 0;
}

void ReadOption(int & argc, char ** & argv, char & crite, dblvec & threshs, double & cutgap, vector<Onode> & otree)
{
  int i, id, idx, cnt;
  char * thc, opt, s[g_bufsize];
  ifstream dendFile;
  map<int, Onode> dend;
  map<int, Onode>::iterator mi;
  Onode curnode;

  cout << "Command line:";
  for (i = 0; i < argc; i++) {
    cout << ' ' << argv[i];
  }
  cout << endl;
  // Remove the executable name
  argv++;
  argc--;

  if (argc == 0) {
    PrintUsage(cerr);
    exit(1);
  }
  while (argv[0][0] == '-') {
    opt = argv[0][1];
    argv++;
    argc--;
    switch (opt) {
    case 'c':
      crite = argv[0][0];
      argv++;
      argc--;
      thc = strtok(argv[0], " ");
      while (thc != NULL) {
	threshs.push_back(atof(thc));
	thc = strtok(NULL, " ");
      }
      argv++;
      argc--;
      break;
    case 'g':
      cutgap = atof(argv[0]);
      argv++;
      argc--;
      break;
    default:
      cerr << "Error: option " << opt << endl;
      PrintUsage(cerr);
      exit(1);
    }
  }

  Mylib::InFopen(dendFile, argv[0]);
  argv++;
  argc--;
  idx = 0;
  cnt = 0;
  while (dendFile >> id >> curnode.left >> curnode.right >> curnode.distance >> curnode.upperb >> curnode.lowerb) {
    cnt++;
    if (id > idx) {
      assert(dend.find(id) == dend.end());
      dend[id] = curnode;
    } else {
      assert(id == idx);
      otree.push_back(curnode);
      idx++;
      while ((mi = dend.find(idx)) != dend.end()) {
	otree.push_back(mi->second);
	dend.erase(mi);
	idx++;
      }
    }
    dendFile.getline(s, g_bufsize);
  }
  dendFile.close();
  cout << "otree\t" << idx << "\ttotal\t" << cnt << "\tlast\t" << otree.back().distance << '\t' << otree.back().upperb << '\t' << otree.back().lowerb << '\t' << otree.back().touse << endl;
}

void PrintOption(ostream & out, const int & argc, char ** const & argv, const char & crite, const dblvec & threshs, const double & cutgap)
{
  int i;
  out << "Remaining command line:";
  for (i = 0; i < argc; i++) {
    out << ' ' << argv[i];
  }
  out << endl;
  out << "crite\t" << crite << endl;
  out << "threshs\t";
  Mylib::PrintJoin(out, threshs, " ");
  out << endl;
  out << "cutgap\t" << cutgap << endl;
}

void PrintUsage(ostream & out)
{
  out << "Usage: cluster" << endl;
  out << " [-c <criterion> <thresholds>]" << endl;
  out << " [-g <cutgap>]" << endl;
  out << " [-a]" << endl;
  out << " <i.8f dend>" << endl;
  out << " <o.4f out>" << endl;
  out << endl;
  out << "<criterion>:" << endl;
  out << " 'c': stop merging when the metric for merging" << endl;
  out << "   clusters drops below threshold" << endl;
  out << "<thresholds>:" << endl;
  out << " space-delimited thresholds. Cluster for each" << endl;
  out << endl;
  out << "<cutgap>: make a cluster if it is thrown" << endl;
  out << "  within <cutgap> away from the threshold" << endl;
  out << "Default: 0. A cluster if in pool at theshold" << endl;
  out << "Note this is different from the ogap in HACO" << endl;
  out << endl;
  out << "<dend>: the dendrogram learned" << endl;
  out << " f1: cluster ID" << endl;
  out << " f2: cluster ID of left child. -1 if none" << endl;
  out << " f3: cluster ID of right child. -1 if none" << endl;
  out << " f4: distance when this cluster is created" << endl;
  out << " f5: ub of distance when the cluster is removed" << endl;
  out << " f6: lb of distance when the cluster is removed" << endl;
  out << "The rest can be anything, such as:" << endl;
  out << " f7: touse when the cluster is removed" << endl;
  out << " f8: size of the cluster" << endl;
  out << endl;
  out << "<out>:" << endl;
  out << " f1: threshold" << endl;
  out << " f2: cluster ID, use f1 of <dend>" << endl;
  out << " f3: size of the cluster" << endl;
  out << " f4: space-delimited cluster members" << endl;
}
